"""Simplified python code documentation."""
from .doc import name_to_str, append_doc, append_plot, append_str, insert_doc, insert_str, insert_eq, table, insert_latex, insert_latex_eq
__all__ = ['name_to_str', 'append_doc', 'append_plot', 'append_str', 'insert_doc',
           'insert_str', 'insert_eq', 'table', 'insert_latex', 'insert_latex_eq']
